/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.battlefieldelements;


import java.awt.Graphics;
import java.awt.Image;
import java.awt.Rectangle;
import java.util.List;

import javax.swing.ImageIcon;

import fb.battlefield.BattleFieldModel;
import fb.util.DX;
import fb.util.Resources;

/**
 * a projectile
 * 
 * @author farfpeuf
 * 
 */
public class Projectile extends Animated {

	// images used to paint projectiles
	private static ImageIcon[] fireballIcons, arrowIcons, spearIcons;

	private static ImageIcon canonBallIcon;

	// origin
	private int x, y;

	// destination
	private int xd, yd;

	// offsets
	private int projectileDx, projectileDy;

	private int step = 16;

	// size
	private int width, length;

	// projectile type
	private char type;

	// current image
	private Image image;

	// orientation
	private double alpha;

	// main target
	private Warrior mainTarget;

	// other targets
	private List otherTargets;

	/**
	 * constructor with multiple targets
	 * 
	 * @param x
	 *            x origin
	 * @param y
	 *            y origin
	 * @param mainTarget
	 *            main target
	 * @param otherTargets
	 *            other targets
	 * @param type
	 *            type of projectile
	 */
	public Projectile(int x, int y, Warrior mainTarget, List otherTargets,
			char type) {
		this(x, y, mainTarget, type);
		this.otherTargets = otherTargets;
	}

	/**
	 * constructor with single target
	 * 
	 * @param x
	 *            x origin
	 * @param y
	 *            y origin
	 * @param target
	 *            target
	 * @param type
	 *            projectile type
	 */
	public Projectile(int x, int y, Warrior target, char type) {
		this.mainTarget = target;
		this.x = x;
		this.y = y;
		this.xd = target.x;
		this.yd = target.y - 5; // tete
		if (!target.mustDie()) {
			xd = xd - 15 + DX.roll(30);
			yd = yd - 15 + DX.roll(30);
		}
		if (target.isflying()) {
			this.yd -= 20;
		}
		this.type = type;
		if (type == 'b') {
			width = 2;
			length = 2;
		} else {

			width = 5;
			length = 5;
		}
		projectileDx = (int) (0 - width / 2);
		projectileDy = (int) (0 - length / 2);
	}

	/**
	 * @return Returns the x.
	 */
	public int getX() {
		return x;
	}

	/**
	 * @return Returns the y.
	 */
	public int getY() {
		return y;
	}

	/**
	 * paint the projectile on the battlefield
	 */
	public void paintImage(Graphics g, int dx, int dy, int scale) {
		image = getImageToPaint(get8baseAngle(alpha), scale < 3);
		g.drawImage(image, (x + projectileDx) * scale + dx, (y + projectileDy)
				* scale + dy, width * scale, length * scale, null);
	}

	/**
	 * get projetile bounds
	 */
	public Rectangle getBounds() {
		int rx = x - width / 2;
		int ry = y - length / 2;
		return new Rectangle(rx, ry, width, length);
	}

	/**
	 * get the image to use for painting
	 * 
	 * @param angle8
	 *            8-base angle
	 * @param small
	 *            use small or big image
	 * @return the image to paint
	 */
	public Image getImageToPaint(int angle8, boolean small) {
		if (type == 'a') {
			return (arrowIcons[angle8 + (small ? 8 : 0)]).getImage();
		} else if (type == 's') {
			return (spearIcons[angle8]).getImage();
		} else if (type == 'b') {
			return canonBallIcon.getImage();
		} else {
			return (fireballIcons[angle8]).getImage();
		}
	}

	/**
	 * animate the projectile (determine next icon used for painting)
	 */
	public void animate() {
		int distx, disty, pasx, pasy;

		distx = Math.abs(xd - x);
		disty = Math.abs(yd - y);
		if (distx > disty) {
			pasx = (int) ((double) step / (double) (1.0 + (double) disty
					/ (double) distx));
			pasy = step - pasx;
		} else {
			pasy = (int) ((double) step / (double) (1.0 + (double) distx
					/ (double) disty));
			pasx = step - pasy;
		}
		if (pasx > distx) {
			pasx = distx;
		}
		if (pasy > disty) {
			pasy = disty;
		}
		if (x > xd) {
			pasx = -pasx;
		}
		if (y > yd) {
			pasy = -pasy;
		}
		alpha = Math.PI + Math.atan2((double) pasx, (double) pasy);
		doMove(pasx, pasy);
		if (x == xd && y == yd) { // fin
			if (mainTarget.mustDie()) {
				mainTarget.animKill();
			}
			if (otherTargets != null) {
				Warrior target;
				for (int i = 0; i < otherTargets.size(); i++) {
					target = (Warrior) otherTargets.get(i);
					if (target.mustDie()) {
						target.animKill();
					}
				}
			}

			BattleFieldModel.getInstance().removeProjectile(this);
		}

	}

	/**
	 * apply specified move
	 * 
	 * @param xOffset
	 *            x Offset
	 * @param yOffset
	 *            y Offset
	 */
	public void doMove(int xOffset, int yOffset) {
		if (xOffset != 0 || yOffset != 0) {
			Rectangle previousBounds = getBounds();
			x += xOffset;
			y += yOffset;
			firePropertyChange(BOUNDS, previousBounds, getBounds());
		}
	}

	/**
	 * convert a angle in radians to an 8-base angle
	 * 
	 * @param theta
	 *            the angle in radians
	 * @return the 8-base angle
	 */
	public int get8baseAngle(double theta) {
		double twoPi = 2 * Math.PI;
		theta = theta % twoPi;
		if (theta < -5.8905) {
			return 0;
		} else if (theta < -5.1051) {
			return 1;
		} else if (theta < -4.3197) {
			return 2;
		} else if (theta < -3.5343) {
			return 3;
		} else if (theta < -2.7489) {
			return 4;
		} else if (theta < -1.9635) {
			return 5;
		} else if (theta < -1.1781) {
			return 6;
		} else if (theta < -0.3927) {
			return 7;
		} else if (theta < 0.3927) {
			return 0;
		} else if (theta < 1.1781) {
			return 1;
		} else if (theta < 1.9635) {
			return 2;
		} else if (theta < 2.7489) {
			return 3;
		} else if (theta < 3.5343) {
			return 4;
		} else if (theta < 4.3197) {
			return 5;
		} else if (theta < 5.1051) {
			return 6;
		} else if (theta < 5.8905) {
			return 7;
		} else {
			return 0;
		}
	}

	/**
	 * load images used to paint projectiles
	 * 
	 */
	public static void loadProjectilesIcons() {
		if (arrowIcons == null) {
			arrowIcons = new ImageIcon[] { Resources.getSign("arrow0"),
					Resources.getSign("arrow1"), Resources.getSign("arrow2"),
					Resources.getSign("arrow3"), Resources.getSign("arrow4"),
					Resources.getSign("arrow5"), Resources.getSign("arrow6"),
					Resources.getSign("arrow7"), Resources.getSign("arrow0s"),
					Resources.getSign("arrow1s"), Resources.getSign("arrow2s"),
					Resources.getSign("arrow3s"), Resources.getSign("arrow4s"),
					Resources.getSign("arrow5s"), Resources.getSign("arrow6s"),
					Resources.getSign("arrow7s") };
		}
		if (fireballIcons == null) {
			fireballIcons = new ImageIcon[] { Resources.getSign("fireball0"),
					Resources.getSign("fireball1"),
					Resources.getSign("fireball2"),
					Resources.getSign("fireball3"),
					Resources.getSign("fireball4"),
					Resources.getSign("fireball5"),
					Resources.getSign("fireball6"),
					Resources.getSign("fireball7"), };
		}
		if (canonBallIcon == null) {
			canonBallIcon = Resources.getSign("canonball");
		}
		if (spearIcons == null) {
			spearIcons = new ImageIcon[] { Resources.getSign("spear0"),
					Resources.getSign("spear1"), Resources.getSign("spear2"),
					Resources.getSign("spear3"), Resources.getSign("spear4"),
					Resources.getSign("spear5"), Resources.getSign("spear6"),
					Resources.getSign("spear7"), };
		}

	}

}