/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.campaign;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Shape;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.List;

import fb.army.hierarchy.Army;
import fb.army.hierarchy.Force;
import fb.main.TimerListener;
import fb.util.FBColors;

/**
 * panel where campaign map is painted
 * 
 * @author farfpeuf
 * 
 */
public class CampaignMapPanel extends HexagonalMapPanel implements
		TimerListener, PropertyChangeListener {

	// PropertyChange delegate
	protected PropertyChangeSupport support;

	// panel is animated every 2 time tics
	private boolean animate;

	/**
	 * actions performed every timer tic
	 * 
	 */
	public void timeTic() {
		// animate every 2 time tics
		animate = !animate;
		if (animate) {
			boolean wait = animate();
			Campaign.getInstance().waitAnimation(wait);
		}
		super.timeTic();
	}

	/**
	 * test if armies are currently moving
	 * 
	 * @return true if one or more army is moving
	 */
	private boolean animate() {
		boolean result = false;
		Force[] forces = Campaign.getInstance().getForces();
		List armies;
		Army army;
		Point oldCoords;
		for (int i = 0; i < forces.length; i++) {
			armies = forces[i].getArmies();
			for (int j = 0; j < armies.size(); j++) {
				army = (Army) armies.get(j);
				oldCoords = army.getAnimationPosition();
				if (army.animate()) {
					result = true;
					refresh(oldCoords);
					refresh(army.getAnimationPosition());
				}
			}
		}
		return result;
	}

	/**
	 * handle mouse click on the map
	 * 
	 * @param mouse
	 *            mouse coordinates
	 */
	public void click(Point mouse) {
		Point hexaClick = getHexaCoord(mouse);
		Campaign.getInstance().click(hexaClick);

	}
	/**
	 * start panel animation
	 */
	public void start() {
		super.start();
		Campaign.getInstance().addPropertyChangeListener(
				Campaign.SELECTED_ARM_PROPERTY, this);
		Campaign.getInstance().addPropertyChangeListener(
				Campaign.MVT_ZONE_PROPERTY, this);
	}

	/**
	 * stop panel animation
	 */
	public void stop() {
		Campaign.getInstance().removePropertyChangeListener(
				Campaign.SELECTED_ARM_PROPERTY, this);
		Campaign.getInstance().removePropertyChangeListener(
				Campaign.MVT_ZONE_PROPERTY, this);
		super.stop();
	}

	/**
	 * paint the armies on the panel
	 * @param g graphics
	 */
	public void paintArmies(Graphics g) {
		Force[] forces = Campaign.getInstance().getForces();
		Graphics2D g2d = (Graphics2D) g;
		List armies;
		Point center;
		Army army;
		for (int i = 0; i < forces.length; i++) {
			armies = forces[i].getArmies();
			for (int j = 0; j < armies.size(); j++) {
				army = (Army) armies.get(j);
				center = getMouseCoord(army.getAnimationPosition());

				Rectangle armRect = new Rectangle(center.x - getRadius() / 2,
						center.y - getRadius() / 2, getRadius(), getRadius());
				if (forces[i] == Campaign.getInstance().currentForce()
						&& !forces[i].isAi() && forces[i].isHandled()) {
					Shape hexagon = new Hexagon(center.x, center.y, getRadius());
					if (army == Campaign.getInstance().getSelectedArmy()) {
						g.setColor(FBColors.secondary);
					} else {
						g.setColor(FBColors.primary);
					}
					g2d.draw(hexagon);
				}

				g.setColor(army.getColor());
				g.fillOval(armRect.x - 2, armRect.y - 2, armRect.width + 4,
						armRect.height + 4);

				Image image = army.getIcon().getImage();
				g.drawImage(image, armRect.x, armRect.y, armRect.width,
						armRect.height, null);

			}
		}
	}

	/**
	 * paint city controls (Army Flags) on the map
	 * @param g graphics
	 */
	public void paintCityControls(Graphics g) {
		Graphics2D g2d = (Graphics2D) g;
		Hashtable controls = Campaign.getInstance().getCityControls();
		Enumeration keys = controls.keys();
		Point coord, center;
		Shape flag;
		int side;
		List forces;
		Force force;
		while (keys.hasMoreElements()) {
			coord = (Point) keys.nextElement();
			side = ((Integer) controls.get(coord)).intValue();
			forces = Campaign.getInstance().getForces(side);
			center = getMouseCoord(coord);

			for (int i = 0; i < forces.size(); i++) {
				force = (Force) forces.get(i);
				flag = new Flag(center.x - 3 * getRadius() / 4 + i * 3,
						center.y + i * 3, getRadius());
				g.setColor(force.getColor());
				g2d.fill(flag);
				g.setColor(Color.BLACK);
				g2d.draw(flag);
			}

		}
	}

	/**
	 * draw the panel
	 */
	protected void draw(Graphics g) {
		super.draw(g);
		g.translate(dx, dy);
		paintArmies(g);
		paintCityControls(g);
		g.translate(-dx, -dy);

	}

	/**
	 * propertyChange managment
	 */
	public void propertyChange(PropertyChangeEvent evt) {
		if (evt.getPropertyName().equals(Army.POSITION)) {
			refresh((Point) evt.getOldValue());
			refresh((Point) evt.getNewValue());
		} else if (evt.getPropertyName().equals(Campaign.SELECTED_ARM_PROPERTY)) {
			Force[] forces = Campaign.getInstance().getForces();
			List armies;
			Army army;
			for (int i = 0; i < forces.length; i++) {
				armies = forces[i].getArmies();
				for (int j = 0; j < armies.size(); j++) {
					army = (Army) armies.get(j);
					refresh(army.getAnimationPosition());
				}
			}

		} else if (evt.getPropertyName().equals(Campaign.MVT_ZONE_PROPERTY)) {
			List oldZone = (List) evt.getOldValue(), newZone = (List) evt
					.getNewValue();
			if (oldZone != null) {
				refresh(oldZone);
			}
			if (newZone != null) {
				refresh(newZone);
			}

		}
	}

	/**
	 * refresh tile
	 * @param hexacoord tile coordinates
	 */
	public synchronized void refresh(Point hexacoord) {
		refresh(getHexaRectangle(hexacoord));
	}
	/**
	 * refresh a list of tiles
	 * @param hexacoord list of tiles coordinates
	 */
	public synchronized void refresh(List hexacoords) {
		for (int i = 0; i < hexacoords.size(); i++) {
			refresh((Point) hexacoords.get(i));
		}
	}


}
