/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.campaign;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Shape;

import javax.swing.ImageIcon;

import fb.util.FBColors;
import fb.util.Resources;

/**
 * hexagonal map painting delegate
 * 
 * @author farfpeuf
 * 
 */
public class HexagonalMapPainter {

	// hexagons default radius
	protected int radius = 10;

	// sinus(pi/3) constant
	public static final float SIN_PI_SUR_3 = (float) (Math.sqrt(3) / 2.0);

	// the map data model
	protected HexagonalMapModel model;

	/**
	 * paint the map
	 * 
	 * @param g
	 *            graphics
	 */
	public void paintMap(Graphics g) {

		Shape oldClip = g.getClip();
		Point limitPoint = getMouseCoord(new Point(model.getMapWidth() - 1,
				model.getMapLength() - 1));
		Rectangle clip = (new Rectangle(radius / 2, 0, limitPoint.x,
				limitPoint.y - 1));

		g.setClip(clip.intersection(oldClip.getBounds()));
		g.setColor(FBColors.background);

		Point center;
		for (int i = 0; i < model.getMapWidth(); i++) {
			for (int j = 0; j < model.getMapLength(); j++) {

				if (model != null) {
					paintTile(g, i, j);
					if (radius > 10) {
						drawTileTransitions(g, i, j);
					}
					paintTileVSets(g, new Point(i, j));
				} else {
					center = getMouseCoord(new Point(i, j));
					g.translate(center.x, center.y);
					g.setColor(getTileColor(i, j));
					g.fillRect(-radius, -radius, radius * 2, radius * 2);
					g.translate(-center.x, -center.y);
				}

			}

		}
		g.setClip(oldClip);
	}

	/**
	 * get specified tile color
	 * 
	 * @param i
	 *            tile row
	 * @param j
	 *            tile column
	 * @return tile color
	 */
	public Color getTileColor(int i, int j) {
		if (model != null) {
			char tile = model.getTile(i, j);
			System.out.println(tile);
			return FBColors.color[tile % (FBColors.color.length - 1)];
		}
		return Color.GRAY;
	}

	/**
	 * paint a tile
	 * 
	 * @param g
	 *            graphics
	 * @param i
	 *            tile row
	 * @param j
	 *            tile column
	 */
	public void paintTile(Graphics g, int i, int j) {
		Point center = getMouseCoord(new Point(i, j));
		char tile = getTile(i, j);
		Image image = Resources.getTileImage(tile).getImage();
		g.drawImage(image, center.x - radius, (int) (center.y - SIN_PI_SUR_3
				* radius), radius * 2, (int) (2 * SIN_PI_SUR_3 * radius), null);
	}

	/**
	 * get tile type
	 * 
	 * @param i
	 *            tile row
	 * @param j
	 *            tile column
	 * @return tile type
	 */
	public char getTile(int i, int j) {
		if (HexagonalMapModel.getInstance().isVillageOrCapital(new Point(i, j))) {
			return HexagonalMapModel.DEFAULT_TILE;
		} else {
			return model.getTile(i, j);
		}
	}

	/**
	 * paint tile vertical sets (buildings ...)
	 * 
	 * @param g
	 *            graphics
	 * @param coord
	 *            tile coordinates
	 */
	private void paintTileVSets(Graphics g, Point coord) {
		if (HexagonalMapModel.getInstance().isVillage(coord)) {
			paintTileVSets(g, coord, "X");
		} else if (HexagonalMapModel.getInstance().isCapital(coord)) {
			paintTileVSets(g, coord, "E");
		}
	}

	/**
	 * 
	 * paint tile vertical sets (buildings ...)
	 * 
	 * @param g
	 *            graphics
	 * @param coord
	 *            tile coordinates
	 * @param vSetName
	 *            set image name
	 */
	private void paintTileVSets(Graphics g, Point coord, String vSetName) {

		Point center = getMouseCoord(coord);
		ImageIcon icon = Resources.getVerticalSetImage(vSetName);
		int height = (int) (2 * SIN_PI_SUR_3 * radius);
		int width = height * icon.getIconWidth() / icon.getIconHeight();

		g.drawImage(icon.getImage(), center.x - width / 2, center.y - height
				/ 2, width, height, null);

	}

	/**
	 * draw transitions with surrounding tiles
	 * @param g graphics
	 * @param i tile row
	 * @param j tile column
	 */
	private void drawTileTransitions(Graphics g, int i, int j) {
		char tile = getTile(i, j);
		int tileLayer = getLayer(tile);
		int[] layers = new int[6];
		int[] transitions = new int[6];
		Point coords, tileCoords = new Point(i, j);
		for (int k = 0; k < layers.length; k++) {
			coords = Hexagon.getTranslatedCoords(tileCoords, 1, k);
			if (coords.x >= 0 && coords.y >= 0
					&& coords.x < model.getMapWidth()
					&& coords.y < model.getMapLength()) {
				layers[k] = getLayer(getTile(coords.x, coords.y));
			} else {
				layers[k] = -1;
			}
		}
		for (int k = transitions.length - 1; k >= 0; k--) {
			if (layers[k] <= tileLayer) {
				transitions[k] = 0;
			} else {

				transitions[k]++;
				if (k != 5 && layers[k + 1] == layers[k]) {
					transitions[k] += transitions[k + 1];
					transitions[k + 1] = 0;

				}
				if (k == 0 && transitions[k] != 6) {
					for (int l = transitions.length - 1; l > 0; l--) {
						if (layers[l] == layers[(l + 1) % 6]) {
							transitions[l] += transitions[(l + 1) % 6];
							transitions[(l + 1) % 6] = 0;
						} else {
							break;
						}

					}

				}

			}
		}
		for (int k = 0; k < transitions.length; k++) {
			if (transitions[k] > 0) {
				if (transitions[k] > 3) {
					drawTileTransition(g, i, j, k, 3, layers[k]);
					drawTileTransition(g, i, j, (k + 3) % 6,
							transitions[k] - 3, layers[k]);
				} else {
					drawTileTransition(g, i, j, k, transitions[k], layers[k]);
				}
			}

		}
	}

	/**
	 * draw transition with surrounding tiles
	 * @param g graphics
	 * @param i tile row
	 * @param j tile column
	 * @param direction transition beggining direction
	 * @param length transition length
	 * @param layer transition layer
	 */
	private void drawTileTransition(Graphics g, int i, int j, int direction,
			int length, int layer) {
		Point center = getMouseCoord(new Point(i, j));

		Image image = Resources.getTileTransition(direction, length, layer);
		g.drawImage(image, center.x - radius, (int) (center.y - SIN_PI_SUR_3
				* radius), radius * 2, (int) (2 * SIN_PI_SUR_3 * radius), null);
	}

	/**
	 * get tile layer
	 * @param tile tile type
	 * @return layer
	 */
	private int getLayer(char tile) {
		switch (tile) {
		case 'e':
			return 0;
		case 'f':
			return 1;
		case 'g':
			return 2;
		case 'b':
			return 3;
		case 'a':
			return 4;
		case 'd':
			return 5;
		case 'h':
			return 6;

		}

		return 0;
	}

	/**
	 * get the map data model
	 * @return the map data model
	 */
	public HexagonalMapModel getModel() {
		return model;
	}

	/**
	 * change the map data model
	 * @param model the new map data model
	 */
	public void setModel(HexagonalMapModel model) {
		this.model = model;
	}

	/**
	 * change hexagons radius (zoom in and out)
	 * @param i the new radius
	 */
	public void setRadius(int i) {
		radius = i;
	}

	/**
	 * get the hexagons radius
	 * @return the radius
	 */
	public int getRadius() {
		return radius;
	}

	/**
	 * convert mouse coordinates to hexagonal (row,column) coordinates
	 * @param mouseCoord the mouse coordinates
	 * @return the corresponding hexagonal coordinates
	 */
	public Point getHexaCoord(Point mouseCoord) {
		Point result = new Point();

		int a = (int) (mouseCoord.x % (1.5 * radius));
		int b = (int) (mouseCoord.x / (1.5 * radius));

		result.x = b;

		if (result.x % 2 == 1) {
			result.y = mouseCoord.y / (int) (2 * SIN_PI_SUR_3 * radius);
		} else {
			result.y = (int) (mouseCoord.y + SIN_PI_SUR_3 * radius)
					/ (int) (2 * SIN_PI_SUR_3 * radius);
		}
		if (a < 0.5f * radius) {
			Point hexaCenter = getMouseCoord(result);
			int c = mouseCoord.x - (hexaCenter.x - radius);
			int d = mouseCoord.y - hexaCenter.y;
			if (d < 0) {
				if (d < -c * Math.sqrt(3)) {
					result.x -= 1;
					if (result.x % 2 == 1) {
						result.y -= 1;
					}
				}

			} else {
				if (d > c * Math.sqrt(3)) {
					result.x -= 1;
					result.y += 1;
					if (result.x % 2 == 1) {
						result.y -= 1;
					}
				}
			}

		}

		return result;
	}

	/**
	 * convert hexagonal coordinates (row,column) to mouse coordinates
	 * @param hexaCoord the hexagonal coordinates
	 * @return the corresponding mouse coordinates
	 */
	public Point getMouseCoord(Point hexaCoord) {
		Point result = new Point();
		int xOffset = (int) (radius * 1.5);
		int yOffset = (int) (SIN_PI_SUR_3 * radius);
		result.x = radius + xOffset * hexaCoord.x;
		result.y = (hexaCoord.y) * 2 * yOffset;
		if (hexaCoord.x % 2 == 1) {
			result.y += yOffset;
		}

		return result;
	}

}
