/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.gui.panels;

import java.awt.Component;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JPanel;
import javax.swing.RepaintManager;
import javax.swing.SwingUtilities;

import fb.main.FB;
import fb.main.TimerListener;
import fb.main.Timer;

/**
 * abstract panel that optimizes refreshing of dirty regions using a timer and
 * handles x and y view offsets when mouse is on the edges of the panel
 * 
 * @author farfpeuf
 * 
 */
public abstract class GamePanel extends JPanel implements TimerListener {

	/**
	 * regions that needs to be repainted
	 */
	protected static List dirtyRegions = new ArrayList();

	/**
	 * is mouse currently on panel edges ?
	 */
	private boolean mouseOnEdges = false;

	/**
	 * current mouse coordinates
	 */
	private Point mouseCoord = new Point();

	/**
	 * timer used to refresh panel
	 */
	private Timer timer;

	/**
	 * x and y view offsets
	 */
	protected int dx = 0, dy = 0;

	/**
	 * use rectangles computations for painting optimization?
	 */
	private static boolean computeRectangles = true;

	/**
	 * the panel mouse handler
	 */
	private GamePanelMouseHandler mouseHanler = new GamePanelMouseHandler();

	/**
	 * constructor enable double buffering
	 */
	public GamePanel() {
		RepaintManager.currentManager(this).setDoubleBufferingEnabled(true);

	}

	/**
	 * register mouse events and start timer
	 * 
	 */
	public void start() {
		Component root = SwingUtilities.getRoot(this);

		root.addMouseMotionListener(mouseHanler);
		root.addMouseListener(mouseHanler);

		if (timer == null) { // si le timer n'est pas actif on le lance
			timer = new Timer(40, this);
			timer.setName("gm panel timer thread");
			timer.start();
		}

	}

	/**
	 * handles x and y view offsets when mouse is on the edges
	 * 
	 */
	protected void testEdgeMove() {
		if (isMouseOnEdges()) {
			boolean clipChanged = false;

			int mouseX = mouseCoord.x;
			int mouseY = mouseCoord.y;

			Component root = SwingUtilities.getRoot(this);
			if (root != null) {

				if (mouseX <= 5) {

					setDx(getDx() + 30);
					clipChanged = true;
				} else if (mouseX >= root.getWidth() - 5) {

					setDx(getDx() - 30);
					clipChanged = true;
				}
				if (mouseY <= 5) {

					setDy(getDy() + 30);
					clipChanged = true;
				} else if (mouseY >= root.getHeight() - 5) {

					setDy(getDy() - 30);
					clipChanged = true;
				}
				if (clipChanged) {
					refreshAll();
				}

			}
		}
	}

	/**
	 * refresh all the panel
	 * 
	 */
	public synchronized void refreshAll() {
		dirtyRegions.add(new Rectangle(0, 0, getWidth(), getHeight()));
	}

	/**
	 * refresh a region of the panel
	 * 
	 * @param dirtyRegion
	 *            the region
	 */
	public synchronized void refresh(Rectangle dirtyRegion) {
		dirtyRegions.add(dirtyRegion);
	}

	/**
	 * action performed every time tic
	 * 
	 */
	public void timeTic() {
		testEdgeMove();
		testRepaint();
	}

	/**
	 * repaint dirty regions if needed
	 * 
	 */
	protected void testRepaint() {
		if (!dirtyRegions.isEmpty()) {
			synchronized (this) {

				Rectangle dirtyRegion;
				if (computeRectangles) {
					Rectangle computedDirtyRegion = new Rectangle();
					for (int i = 0; i < dirtyRegions.size(); i++) {
						dirtyRegion = (Rectangle) dirtyRegions.get(i);
						if (dirtyRegion != null) {

							SwingUtilities.computeUnion(dirtyRegion.x,
									dirtyRegion.y, dirtyRegion.width,
									dirtyRegion.height, computedDirtyRegion);
						}
					}
					repaint(computedDirtyRegion);
				} else {
					for (int i = 0; i < dirtyRegions.size(); i++) {
						dirtyRegion = (Rectangle) dirtyRegions.get(i);
						repaint(dirtyRegion);
					}
				}

				dirtyRegions.clear();
			}

		}
	}

	/**
	 * unregister mouse events and stop timer
	 * 
	 */
	public void stop() {

		if (timer != null) {
			timer.safeStop();
		}
		timer = null;
		Component root = SwingUtilities.getRoot(this);
		if (root != null) {

			root.removeMouseMotionListener(mouseHanler);
			root.removeMouseListener(mouseHanler);
		}
	}

	// overidden for custom painting
	public synchronized void paintComponent(Graphics g) {
		super.paintComponent(g);
		draw(g);
	}

	/**
	 * custom paintings
	 * 
	 * @param g
	 *            raphics
	 */
	protected abstract void draw(Graphics g);

	/**
	 * @return Returns the dx.
	 */
	public int getDx() {
		return dx;
	}

	/**
	 * @param dx
	 *            The dx to set.
	 */
	public void setDx(int dx) {
		this.dx = dx;
	}

	/**
	 * @return Returns the dy.
	 */
	public int getDy() {
		return dy;
	}

	/**
	 * @param dy
	 *            The dy to set.
	 */
	public void setDy(int dy) {
		this.dy = dy;
	}

	/**
	 * @return true if mouse is on the edges of the panel
	 */
	public boolean isMouseOnEdges() {
		if (FB.getInstance() != null && FB.getInstance().isMini()) {
			return false;
		}
		return mouseOnEdges;

	}

	/**
	 * panel mouse actions and motions listener
	 * 
	 * @author farfpeuf
	 * 
	 */
	private class GamePanelMouseHandler implements MouseMotionListener,
			MouseListener {
		// MouseListener implementation
		public void mouseClicked(MouseEvent e) {
		}

		public void mouseEntered(MouseEvent e) {
			setMouseOnEdges(true);
		}

		public void mouseExited(MouseEvent e) {
			setMouseOnEdges(false);
		}

		public void mousePressed(MouseEvent e) {
		}

		public void mouseReleased(MouseEvent e) {
		}

		// MouseMotionListener implementation
		public void mouseDragged(MouseEvent e) {
		}

		public void mouseMoved(MouseEvent e) {
			setMouseCoord(e.getPoint());
		}
	}

	/**
	 * @return Returns the mouseCoord.
	 */
	public Point getMouseCoord() {
		return mouseCoord;
	}

	/**
	 * @param mouseCoord
	 *            The mouseCoord to set.
	 */
	public void setMouseCoord(Point mouseCoord) {
		this.mouseCoord = mouseCoord;
	}

	/**
	 * @param mouseOnEdges
	 *            The mouseOnEdges to set.
	 */
	public void setMouseOnEdges(boolean mouseOnEdges) {
		this.mouseOnEdges = mouseOnEdges;
	}

}