/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.gui.panels;

import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.DefaultCellEditor;
import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.event.ListDataEvent;
import javax.swing.event.ListDataListener;
import javax.swing.table.DefaultTableColumnModel;
import javax.swing.table.TableColumn;

import fb.battlefield.MapModel;
import fb.gui.widgets.AIEditor;
import fb.gui.widgets.AIRenderer;
import fb.gui.widgets.ChoiceRenderer;
import fb.gui.widgets.FBComboBox;
import fb.gui.widgets.StringRenderer;
import fb.player.PlayersTableModel;
import fb.util.ArrayListModel;
import fb.util.ComboArrayListModel;
import fb.util.FBColors;
import fb.util.FileArrayListModel;
import fb.util.PercentageLayout;
import fb.util.Resources;

/**
 * panel containing a table of players
 * 
 * @author farfpeuf
 * 
 */
public class PlayersTable extends JPanel implements ListDataListener {
	/**
	 * army files directory
	 */
	private static String armsdir = "armies";

	/**
	 * the table
	 */
	private JTable table;

	/**
	 * combo to choose player side
	 */
	private FBComboBox sideChooser;

	/**
	 * combo to choose player army
	 */
	private FBComboBox armChooser;

	/**
	 * set the available sides
	 * 
	 * @param sides
	 *            the available sides registerable list
	 */
	public void setAvailableSides(ArrayListModel sides) {
		sideChooser.setModel(new ComboArrayListModel(sides));
		sides.addListDataListener(this);
	}

	/**
	 * set the available armies
	 * 
	 * @param sides
	 *            the available armies registerable list
	 */
	public void setAvailableArmies(ArrayListModel list) {
		armChooser.setModel(new ComboArrayListModel(list));
	}

	// ListDataListener implementation

	public void intervalAdded(ListDataEvent e) {
		dispatchSides();
	}

	public void intervalRemoved(ListDataEvent e) {
		dispatchSides();
	}

	public void contentsChanged(ListDataEvent e) {
	}

	/**
	 * build table columns
	 * 
	 * @param editable
	 *            is table editable ?
	 */
	private void buildColumns(boolean editable) {
		TableColumn column0 = new TableColumn(0);
		TableColumn column1 = new TableColumn(1);
		TableColumn column2 = new TableColumn(2);
		TableColumn column3 = new TableColumn(3);

		setColumnWidths(column0, 100);
		setColumnWidths(column1, 20);
		setColumnWidths(column2, 100);
		setColumnWidths(column3, 40);

		column0.setHeaderValue(Resources.getString("player"));
		column1.setHeaderValue(Resources.getString("AI"));
		column2.setHeaderValue(Resources.getString("Army"));
		column3.setHeaderValue(Resources.getString("Side"));

		table.addColumn(column0);
		table.addColumn(column1);
		table.addColumn(column2);
		table.addColumn(column3);

		if (editable) {
			JTextField playerNamefield;
			sideChooser = new FBComboBox();
			sideChooser.setToolTipText(Resources.getString("side.tt"));
			armChooser = new FBComboBox();

			armChooser.setOpaque(false);
			playerNamefield = new JTextField();
			playerNamefield.setOpaque(false);
			playerNamefield.setForeground(FBColors.standardText);
			playerNamefield.setHorizontalAlignment(JTextField.CENTER);
			column0.setCellRenderer(new StringRenderer());
			column1.setCellRenderer(new AIRenderer());
			column2.setCellRenderer(new ChoiceRenderer());
			column3.setCellRenderer(new ChoiceRenderer());
			DefaultCellEditor playerNameEditor = new DefaultCellEditor(
					playerNamefield);
			playerNameEditor.setClickCountToStart(1);
			column0.setCellEditor(playerNameEditor);
			column1.setCellEditor(new AIEditor());
			column2.setCellEditor(new DefaultCellEditor(armChooser));
			column3.setCellEditor(new DefaultCellEditor(sideChooser));

		} else {
			table.setEnabled(false);
			column0.setCellRenderer(new StringRenderer());
			column1.setCellRenderer(new AIRenderer());
			column2.setCellRenderer(new StringRenderer());
			column3.setCellRenderer(new StringRenderer());
		}

	}

	/**
	 * constructor
	 * 
	 * @param editable
	 *            is table editable ?
	 * @param model
	 *            the table model
	 */
	public PlayersTable(boolean editable, PlayersTableModel model) {
		this(editable, model, MapModel.getInstance().getSides(),
				FileArrayListModel.getInstance(armsdir));
	}

	/**
	 * 
	 * @param editable
	 *            is table editable ?
	 * @param model
	 *            the table model
	 * @param availableSides
	 *            the available sides
	 * @param availableArmies
	 *            the available armies
	 */
	public PlayersTable(boolean editable, PlayersTableModel model,
			ArrayListModel availableSides, ArrayListModel availableArmies) {
		setLayout(new PercentageLayout());
		setBackground(FBColors.background);

		table = new JTable(model, new DefaultTableColumnModel());
		buildColumns(editable);
		if (editable) {
			setAvailableSides(availableSides);
			setAvailableArmies(availableArmies);
		}
		table.setOpaque(false);
		table.setAutoResizeMode(JTable.AUTO_RESIZE_ALL_COLUMNS);
		table.setBorder(BorderFactory.createLineBorder(FBColors.darkBorder));
		add(table.getTableHeader(), new PercentageLayout.Constraint(0, 0, 100, 15));
		add(table, new PercentageLayout.Constraint(0, 15, 100, 85));
		validate();
		repaint();

	}

	/**
	 * set table column width
	 * 
	 * @param column
	 *            the column
	 * @param width
	 *            the new width
	 */
	private void setColumnWidths(TableColumn column, int width) {
		column.setWidth(width);
		column.setPreferredWidth(width);
		column.setMaxWidth(Integer.MAX_VALUE);
		column.setMinWidth(20);
	}

	/**
	 * add a player to table
	 * 
	 * @param name
	 *            player name
	 * @param army
	 *            player army
	 * @param isia
	 *            is player human or computer
	 * @param side
	 *            player side
	 */
	public void addPlayer(String name, String army, boolean isia, int side) {
		((PlayersTableModel) table.getModel())
				.addPlayer(name, army, isia, side);
	}

	/**
	 * remove player from table
	 * 
	 * @param row
	 *            player row
	 */
	public void removePlayer(int row) {
		stopEdition();
		((PlayersTableModel) table.getModel()).removePlayer(row);
	}

	/**
	 * get player side
	 * 
	 * @param row
	 *            player table row
	 * @return player side
	 */
	public int getPlayerSide(int row) {
		stopEdition();
		String side = (String) table.getValueAt(row, 3);
		return side.charAt(0) - 'A';
	}

	/**
	 * get player name
	 * 
	 * @param row
	 *            player table row
	 * @return player name
	 */
	public String getPlayerName(int row) {
		stopEdition();
		return (String) table.getValueAt(row, 0);
	}

	/**
	 * get player army name
	 * 
	 * @param row
	 *            player table row
	 * @return player army name
	 */
	public String getPlayerArmy(int row) {
		stopEdition();
		return (String) table.getValueAt(row, 2);
	}

	/**
	 * is player computer or human
	 * 
	 * @param row
	 *            player table row
	 * @return player true if player is computer
	 */
	public boolean getPlayerAI(int row) {
		stopEdition();
		return ((Boolean) table.getValueAt(row, 1)).booleanValue();
	}

	/**
	 * change player side
	 * 
	 * @param row
	 *            the player row
	 * @param side
	 *            the new side
	 */
	public void setPlayerSide(int row, int side) {
		table.setValueAt("" + (char) ('A' + side), row, 3);
	}

	/**
	 * change player name
	 * 
	 * @param row
	 *            the player row
	 * @param side
	 *            the new name
	 */
	public void setPlayerName(int row, String name) {
		table.setValueAt(name, row, 0);
	}

	/**
	 * change player army
	 * 
	 * @param row
	 *            the player row
	 * @param side
	 *            the new army name
	 */
	public void setPlayerArmy(int row, String army) {
		table.setValueAt(army, row, 2);
	}

	/**
	 * change player ia
	 * 
	 * @param row
	 *            the player row
	 * @param side
	 *            the new ai
	 */
	public void setPlayerAI(int row, boolean ai) {
		table.setValueAt(new Boolean(ai), row, 1);
	}

	/**
	 * @return the number of players
	 */
	public int getNBPlayers() {
		return table.getRowCount();
	}

	/**
	 * stop table edition
	 * 
	 */
	public void stopEdition() {
		if (table.getCellEditor() != null) {
			table.getCellEditor().stopCellEditing();
		}
	}

	/**
	 * dispatch sides among players
	 * 
	 */
	protected void dispatchSides() {
		if (sideChooser.getItemCount() > 0) {
			List sides = ((ComboArrayListModel) sideChooser.getModel())
					.getAvailableElements();
			for (int i = 0; i < table.getRowCount(); i++) {
				table.setValueAt(sides.get(i % sides.size()), i, 3);
			}
		}
	}

}